@@ -1,5 +1,5 @@
 # Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
-# For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt
+# For details: https://github.com/coveragepy/coveragepy/blob/main/NOTICE.txt
 
 """Central control stuff for coverage.py."""
 
@@ -8,6 +8,7 @@ from __future__ import annotations
 import atexit
 import collections
 import contextlib
+import datetime
 import functools
 import os
 import os.path
@@ -19,12 +20,12 @@ import time
 import warnings
 from collections.abc import Iterable, Iterator
 from types import FrameType
-from typing import IO, Any, Callable, Union, cast
+from typing import IO, Any, Callable, cast
 
 from coverage import env
 from coverage.annotate import AnnotateReporter
 from coverage.collector import Collector
-from coverage.config import CoverageConfig, deserialize_config, read_coverage_config
+from coverage.config import CoverageConfig, read_coverage_config
 from coverage.context import combine_context_switchers, should_start_context_test_function
 from coverage.core import CTRACER_FILE, Core
 from coverage.data import CoverageData, combine_parallel_data
@@ -67,6 +68,7 @@ from coverage.types import (
     TLineNo,
     TMorf,
 )
+from coverage.version import __url__
 from coverage.xmlreport import XmlReporter
 
 os = isolate_module(os)
@@ -297,7 +299,7 @@ class Coverage(TConfigurable):
         self._debug: DebugControl = NoDebugging()
         self._inorout: InOrOut | None = None
         self._plugins: Plugins = Plugins()
-        self._plugin_override = cast(Union[Iterable[TCoverageInit], None], plugins)
+        self._plugin_override = cast(Iterable[TCoverageInit] | None, plugins)
         self._data: CoverageData | None = None
         self._data_to_close: list[CoverageData] = []
         self._core: Core | None = None
@@ -320,7 +322,7 @@ class Coverage(TConfigurable):
 
         # Build our configuration from a number of sources.
         if isinstance(config_file, str) and config_file.startswith(CONFIG_DATA_PREFIX):
-            self.config = deserialize_config(config_file[len(CONFIG_DATA_PREFIX) :])
+            self.config = CoverageConfig.deserialize(config_file[len(CONFIG_DATA_PREFIX) :])
         else:
             if not isinstance(config_file, bool):
                 config_file = os.fspath(config_file)
@@ -413,22 +415,24 @@ class Coverage(TConfigurable):
         wrote_any = False
         with self._debug.without_callers():
             if self._debug.should("config"):
-                config_info = self.config.debug_info()
-                write_formatted_info(self._debug.write, "config", config_info)
+                write_formatted_info(self._debug.write, "config", self.config.debug_info())
                 wrote_any = True
 
             if self._debug.should("sys"):
                 write_formatted_info(self._debug.write, "sys", self.sys_info())
                 for plugin in self._plugins:
                     header = "sys: " + plugin._coverage_plugin_name
-                    info = plugin.sys_info()
-                    write_formatted_info(self._debug.write, header, info)
+                    write_formatted_info(self._debug.write, header, plugin.sys_info())
                 wrote_any = True
 
             if self._debug.should("pybehave"):
                 write_formatted_info(self._debug.write, "pybehave", env.debug_info())
                 wrote_any = True
 
+            if self._debug.should("sqlite"):
+                write_formatted_info(self._debug.write, "sqlite", CoverageData.sys_info())
+                wrote_any = True
+
         if wrote_any:
             write_formatted_info(self._debug.write, "end", ())
 
@@ -479,7 +483,7 @@ class Coverage(TConfigurable):
 
         self._warnings.append(msg)
         if slug:
-            msg = f"{msg} ({slug})"
+            msg = f"{msg} ({slug}); see {__url__}/messages.html#warning-{slug}"
         if self._debug.should("pid"):
             msg = f"[{os.getpid()}] {msg}"
         warnings.warn(msg, category=CoverageWarning, stacklevel=2)
@@ -559,7 +563,7 @@ class Coverage(TConfigurable):
     def _init_for_start(self) -> None:
         """Initialization for start()"""
         # Construct the collector.
-        concurrency: list[str] = self.config.concurrency or []
+        concurrency: list[str] = self.config.concurrency
         if "multiprocessing" in concurrency:
             if self.config.config_file is None:
                 raise ConfigError("multiprocessing requires a configuration file")
@@ -581,6 +585,7 @@ class Coverage(TConfigurable):
 
         self._core = Core(
             warn=self._warn,
+            debug=(self._debug if self._debug.should("core") else None),
             config=self.config,
             dynamic_contexts=(should_start_context is not None),
             metacov=self._metacov,
@@ -793,7 +798,7 @@ class Coverage(TConfigurable):
     def clear_exclude(self, which: str = "exclude") -> None:
         """Clear the exclude list."""
         self._init()
-        setattr(self.config, which + "_list", [])
+        setattr(self.config, f"{which}_list", [])
         self._exclude_regex_stale()
 
     def exclude(self, regex: str, which: str = "exclude") -> None:
@@ -812,7 +817,7 @@ class Coverage(TConfigurable):
 
         """
         self._init()
-        excl_list = getattr(self.config, which + "_list")
+        excl_list = getattr(self.config, f"{which}_list")
         excl_list.append(regex)
         self._exclude_regex_stale()
 
@@ -823,7 +828,7 @@ class Coverage(TConfigurable):
     def _exclude_regex(self, which: str) -> str:
         """Return a regex string for the given exclusion list."""
         if which not in self._exclude_re:
-            excl_list = getattr(self.config, which + "_list")
+            excl_list = getattr(self.config, f"{which}_list")
             self._exclude_re[which] = join_regex(excl_list)
         return self._exclude_re[which]
 
@@ -835,7 +840,7 @@ class Coverage(TConfigurable):
 
         """
         self._init()
-        return cast(list[str], getattr(self.config, which + "_list"))
+        return cast(list[str], getattr(self.config, f"{which}_list"))
 
     def save(self) -> None:
         """Save the collected coverage data to the data file."""
@@ -1064,6 +1069,7 @@ class Coverage(TConfigurable):
         if not isinstance(morfs, (list, tuple, set)):
             morfs = [morfs]  # type: ignore[list-item]
 
+        morfs = sorted(morfs, key=lambda m: m if isinstance(m, str) else m.__name__)
         return [(self._get_file_reporter(morf), morf) for morf in morfs]
 
     def _prepare_data_for_reporting(self) -> None:
@@ -1242,8 +1248,7 @@ class Coverage(TConfigurable):
             precision=precision,
         ):
             reporter = HtmlReporter(self)
-            ret = reporter.report(morfs)
-            return ret
+            return reporter.report(morfs)
 
     def xml_report(
         self,
@@ -1393,13 +1398,12 @@ class Coverage(TConfigurable):
             ("path", sys.path),
             ("environment", [f"{k} = {v}" for k, v in relevant_environment_display(os.environ)]),
             ("command_line", " ".join(getattr(sys, "argv", ["-none-"]))),
+            ("time", f"{datetime.datetime.now():%Y-%m-%d %H:%M:%S}"),
         ]
 
         if self._inorout is not None:
             info.extend(self._inorout.sys_info())
 
-        info.extend(CoverageData.sys_info())
-
         return info
 
 
@@ -1444,7 +1448,7 @@ def process_startup(*, force: bool = False) -> Coverage | None:
     # flag (an attribute on this function) to indicate that coverage.py has
     # already been started, so we can avoid doing it twice.
     #
-    # https://github.com/nedbat/coveragepy/issues/340 has more details.
+    # https://github.com/coveragepy/coveragepy/issues/340 has more details.
 
     if not force and hasattr(process_startup, "coverage"):
         # We've annotated this function before, so we must have already
