@@ -1,5 +1,5 @@
 # Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
-# For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt
+# For details: https://github.com/coveragepy/coveragepy/blob/main/NOTICE.txt
 
 """File wrangling."""
 
@@ -12,19 +12,19 @@ import os.path
 import posixpath
 import re
 import sys
-
-from typing import Callable, Dict, Iterable, List, Optional, Tuple
+from collections.abc import Iterable
+from typing import Callable
 
 from coverage import env
 from coverage.exceptions import ConfigError
 from coverage.misc import human_sorted, isolate_module, join_regex
 
-
 os = isolate_module(os)
 
 
 RELATIVE_DIR: str = ""
-CANONICAL_FILENAME_CACHE: Dict[str, str] = {}
+CANONICAL_FILENAME_CACHE: dict[str, str] = {}
+
 
 def set_relative_directory() -> None:
     """Set the directory that `relative_filename` will be relative to."""
@@ -58,7 +58,7 @@ def relative_filename(filename: str) -> str:
     """
     fnorm = os.path.normcase(filename)
     if fnorm.startswith(RELATIVE_DIR):
-        filename = filename[len(RELATIVE_DIR):]
+        filename = filename[len(RELATIVE_DIR) :]
     return filename
 
 
@@ -73,7 +73,7 @@ def canonical_filename(filename: str) -> str:
         if not os.path.isabs(filename):
             for path in [os.curdir] + sys.path:
                 if path is None:
-                    continue # type: ignore
+                    continue  # type: ignore[unreachable]
                 f = os.path.join(path, filename)
                 try:
                     exists = os.path.exists(f)
@@ -87,8 +87,6 @@ def canonical_filename(filename: str) -> str:
     return CANONICAL_FILENAME_CACHE[filename]
 
 
-MAX_FLAT = 100
-
 def flat_rootname(filename: str) -> str:
     """A base for a flat file name to correspond to this file.
 
@@ -96,22 +94,25 @@ def flat_rootname(filename: str) -> str:
     the same directory, but need to differentiate same-named files from
     different directories.
 
-    For example, the file a/b/c.py will return 'd_86bbcbe134d28fd2_c_py'
+    For example, the file a/b/c.py will return 'z_86bbcbe134d28fd2_c_py'
 
     """
     dirname, basename = ntpath.split(filename)
     if dirname:
-        fp = hashlib.new("sha3_256", dirname.encode("UTF-8")).hexdigest()[:16]
-        prefix = f"d_{fp}_"
+        fp = hashlib.new(
+            "sha3_256",
+            dirname.encode("UTF-8"),
+            usedforsecurity=False,
+        ).hexdigest()[:16]
+        prefix = f"z_{fp}_"
     else:
         prefix = ""
     return prefix + basename.replace(".", "_")
 
 
 if env.WINDOWS:
-
-    _ACTUAL_PATH_CACHE: Dict[str, str] = {}
-    _ACTUAL_PATH_LIST_CACHE: Dict[str, List[str]] = {}
+    _ACTUAL_PATH_CACHE: dict[str, str] = {}
+    _ACTUAL_PATH_LIST_CACHE: dict[str, list[str]] = {}
 
     def actual_path(path: str) -> str:
         """Get the actual path of `path`, including the correct case."""
@@ -146,6 +147,7 @@ if env.WINDOWS:
         return actpath
 
 else:
+
     def actual_path(path: str) -> str:
         """The actual path for non-Windows platforms."""
         return path
@@ -156,14 +158,14 @@ def abs_file(path: str) -> str:
     return actual_path(os.path.abspath(os.path.realpath(path)))
 
 
-def zip_location(filename: str) -> Optional[Tuple[str, str]]:
+def zip_location(filename: str) -> tuple[str, str] | None:
     """Split a filename into a zipfile / inner name pair.
 
     Only return a pair if the zipfile exists.  No check is made if the inner
     name is in the zipfile.
 
     """
-    for ext in ['.zip', '.whl', '.egg', '.pex']:
+    for ext in [".zip", ".whl", ".egg", ".pex", ".par"]:
         zipbase, extension, inner = filename.partition(ext + sep(filename))
         if extension:
             zipfile = zipbase + ext
@@ -187,9 +189,7 @@ def source_exists(path: str) -> bool:
 
 def python_reported_file(filename: str) -> str:
     """Return the string as Python would describe this file name."""
-    if env.PYBEHAVIOR.report_absolute_files:
-        filename = os.path.abspath(filename)
-    return filename
+    return os.path.abspath(filename)
 
 
 def isabs_anywhere(filename: str) -> bool:
@@ -197,7 +197,7 @@ def isabs_anywhere(filename: str) -> bool:
     return ntpath.isabs(filename) or posixpath.isabs(filename)
 
 
-def prep_patterns(patterns: Iterable[str]) -> List[str]:
+def prep_patterns(patterns: Iterable[str]) -> list[str]:
     """Prepare the file patterns for use in a `GlobMatcher`.
 
     If a pattern starts with a wildcard, it is used as a pattern
@@ -209,9 +209,8 @@ def prep_patterns(patterns: Iterable[str]) -> List[str]:
     """
     prepped = []
     for p in patterns or []:
-        if p.startswith(("*", "?")):
-            prepped.append(p)
-        else:
+        prepped.append(p)
+        if not p.startswith(("*", "?")):
             prepped.append(abs_file(p))
     return prepped
 
@@ -224,16 +223,16 @@ class TreeMatcher:
     somewhere in a subtree rooted at one of the directories.
 
     """
+
     def __init__(self, paths: Iterable[str], name: str = "unknown") -> None:
-        self.original_paths: List[str] = human_sorted(paths)
-        #self.paths = list(map(os.path.normcase, paths))
+        self.original_paths: list[str] = human_sorted(paths)
         self.paths = [os.path.normcase(p) for p in paths]
         self.name = name
 
     def __repr__(self) -> str:
         return f"<TreeMatcher {self.name} {self.original_paths!r}>"
 
-    def info(self) -> List[str]:
+    def info(self) -> list[str]:
         """A list of strings for displaying when dumping state."""
         return self.original_paths
 
@@ -253,14 +252,15 @@ class TreeMatcher:
 
 class ModuleMatcher:
     """A matcher for modules in a tree."""
-    def __init__(self, module_names: Iterable[str], name:str = "unknown") -> None:
+
+    def __init__(self, module_names: Iterable[str], name: str = "unknown") -> None:
         self.modules = list(module_names)
         self.name = name
 
     def __repr__(self) -> str:
         return f"<ModuleMatcher {self.name} {self.modules!r}>"
 
-    def info(self) -> List[str]:
+    def info(self) -> list[str]:
         """A list of strings for displaying when dumping state."""
         return self.modules
 
@@ -273,7 +273,7 @@ class ModuleMatcher:
             if module_name.startswith(m):
                 if module_name == m:
                     return True
-                if module_name[len(m)] == '.':
+                if module_name[len(m)] == ".":
                     # This is a module in the package
                     return True
 
@@ -282,6 +282,7 @@ class ModuleMatcher:
 
 class GlobMatcher:
     """A matcher for files by file name pattern."""
+
     def __init__(self, pats: Iterable[str], name: str = "unknown") -> None:
         self.pats = list(pats)
         self.re = globs_to_regex(self.pats, case_insensitive=env.WINDOWS)
@@ -290,7 +291,7 @@ class GlobMatcher:
     def __repr__(self) -> str:
         return f"<GlobMatcher {self.name} {self.pats!r}>"
 
-    def info(self) -> List[str]:
+    def info(self) -> list[str]:
         """A list of strings for displaying when dumping state."""
         return self.pats
 
@@ -301,8 +302,7 @@ class GlobMatcher:
 
 def sep(s: str) -> str:
     """Find the path separator used in this string, or os.sep if none."""
-    sep_match = re.search(r"[\\/]", s)
-    if sep_match:
+    if sep_match := re.search(r"[\\/]", s):
         the_sep = sep_match[0]
     else:
         the_sep = os.sep
@@ -311,6 +311,7 @@ def sep(s: str) -> str:
 
 # Tokenizer for _glob_to_regex.
 # None as a sub means disallowed.
+# fmt: off
 G2RX_TOKENS = [(re.compile(rx), sub) for rx, sub in [
     (r"\*\*\*+", None),             # Can't have ***
     (r"[^/]+\*\*+", None),          # Can't have x**
@@ -327,19 +328,20 @@ G2RX_TOKENS = [(re.compile(rx), sub) for rx, sub in [
     (r"[\[\]]", None),              # Can't have single square brackets
     (r".", r"\\\g<0>"),             # Anything else is escaped to be safe
 ]]
+# fmt: on
+
 
 def _glob_to_regex(pattern: str) -> str:
     """Convert a file-path glob pattern into a regex."""
     # Turn all backslashes into slashes to simplify the tokenizer.
     pattern = pattern.replace("\\", "/")
     if "/" not in pattern:
-        pattern = "**/" + pattern
+        pattern = f"**/{pattern}"
     path_rx = []
     pos = 0
     while pos < len(pattern):
-        for rx, sub in G2RX_TOKENS:                     # pragma: always breaks
-            m = rx.match(pattern, pos=pos)
-            if m:
+        for rx, sub in G2RX_TOKENS:  # pragma: always breaks
+            if m := rx.match(pattern, pos=pos):
                 if sub is None:
                     raise ConfigError(f"File pattern can't include {m[0]!r}")
                 path_rx.append(m.expand(sub))
@@ -390,13 +392,14 @@ class PathAliases:
     map a path through those aliases to produce a unified path.
 
     """
+
     def __init__(
         self,
-        debugfn: Optional[Callable[[str], None]] = None,
+        debugfn: Callable[[str], None] | None = None,
         relative: bool = False,
     ) -> None:
         # A list of (original_pattern, regex, result)
-        self.aliases: List[Tuple[str, re.Pattern[str], str]] = []
+        self.aliases: list[tuple[str, re.Pattern[str], str]] = []
         self.debugfn = debugfn or (lambda msg: 0)
         self.relative = relative
         self.pprinted = False
@@ -433,7 +436,7 @@ class PathAliases:
         # The pattern is meant to match a file path.  Let's make it absolute
         # unless it already is, or is meant to match any prefix.
         if not self.relative:
-            if not pattern.startswith('*') and not isabs_anywhere(pattern + pattern_sep):
+            if not pattern.startswith("*") and not isabs_anywhere(pattern + pattern_sep):
                 pattern = abs_file(pattern)
         if not pattern.endswith(pattern_sep):
             pattern += pattern_sep
@@ -446,7 +449,7 @@ class PathAliases:
         result = result.rstrip(r"\/") + result_sep
         self.aliases.append((original_pattern, regex, result))
 
-    def map(self, path: str, exists:Callable[[str], bool] = source_exists) -> str:
+    def map(self, path: str, exists: Callable[[str], bool] = source_exists) -> str:
         """Map `path` through the aliases.
 
         `path` is checked against all of the patterns.  The first pattern to
@@ -470,8 +473,7 @@ class PathAliases:
             self.pprinted = True
 
         for original_pattern, regex, result in self.aliases:
-            m = regex.match(path)
-            if m:
+            if m := regex.match(path):
                 new = path.replace(m[0], result)
                 new = new.replace(sep(path), sep(result))
                 if not self.relative:
@@ -481,18 +483,21 @@ class PathAliases:
                     new = new[2:]
                 if not exists(new):
                     self.debugfn(
-                        f"Rule {original_pattern!r} changed {path!r} to {new!r} " +
-                        "which doesn't exist, continuing"
+                        f"Rule {original_pattern!r} changed {path!r} to {new!r} "
+                        + "which doesn't exist, continuing",
                     )
                     continue
                 self.debugfn(
-                    f"Matched path {path!r} to rule {original_pattern!r} -> {result!r}, " +
-                    f"producing {new!r}"
+                    f"Matched path {path!r} to rule {original_pattern!r} -> {result!r}, "
+                    + f"producing {new!r}",
                 )
                 return new
 
         # If we get here, no pattern matched.
 
+        if self.relative:
+            path = relative_filename(path)
+
         if self.relative and not isabs_anywhere(path):
             # Auto-generate a pattern to implicitly match relative files
             parts = re.split(r"[/\\]", path)
@@ -504,7 +509,7 @@ class PathAliases:
                 # Only add a new pattern if we don't already have this pattern.
                 if not any(p == pattern for p, _, _ in self.aliases):
                     self.debugfn(
-                        f"Generating rule: {pattern!r} -> {result!r} using regex {regex_pat!r}"
+                        f"Generating rule: {pattern!r} -> {result!r} using regex {regex_pat!r}",
                     )
                     self.aliases.append((pattern, re.compile(regex_pat), result))
                     return self.map(path, exists=exists)
